#define RC_OK ((krc == KRB5_SUCCESS) && (prc == PAM_SUCCESS))
#define PASSWORD_CHANGING_SERVICE "kadmin/changepw"
{
	krb5_context context = NULL;
	krb5_principal principal;
	char *user = NULL, *authtok = NULL, *old_authtok = NULL;
	int prc = PAM_SUCCESS, krc = KRB5_SUCCESS;
	krb5_creds creds;

	if (krb5_init_secure_context(&context) != KRB5_SUCCESS) {
		prc = PAM_SYSTEM_ERR;
	}

        /* Read in the configuration. */
        if (RC_OK) {
                if ((config = get_config(context, argc, argv))) {
                        prc = PAM_SYSTEM_ERR;
                }
        }
        DEBUG("pam_sm_acct_mgmt() called");

        /* Ask libpam for the user's name, and bail if she has none. */
        prc = pam_get_item(pamh, PAM_USER, (const void**)&user);
        if (user == NULL) {
                prc = PAM_USER_UNKNOWN;
        } else {
                krc = krb5_parse_name(context, user, &principal);
                if (krc != KRB5_SUCCESS) {
                        CRIT("%s", error_message(krc));
                }
        }

        /* Read the old password.  It's okay if this fails. */
        if (RC_OK) {
		pam_get_item(pamh, PAM_OLDAUTHTOK, (const void**) &old_authtok);
		pam_get_item(pamh, PAM_AUTHTOK, (const void**) &authtok);
        }
        /* flush out the case where the user has no Kerberos principal, and
           avoid a spurious, potentially confusing password prompt */
        if (RC_OK) {
                krc = krb5_get_init_creds_password(context,
                                                   &creds,
                                                   principal,
                                                   (char*)user,
                                                   NULL,
                                                   NULL,
                                                   0,
                                                   PASSWORD_CHANGING_SERVICE,
                                                   &config->creds_opt);
                if (krc == KRB5_SUCCESS) {
                        DEBUG("user exists, but users's password is equal to "
                              "user's name -- this should be changed");
                } else {
                        if (krc == KRB5KDC_ERR_C_PRINCIPAL_UNKNOWN) {
                                DEBUG("user does not have a Kerberos "
                                      "principal");
                                prc = PAM_USER_UNKNOWN;
                        } else
                        if (krc == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN) {
                                DEBUG("password-changing service does not "
                                      "exist?!?!?");
                                prc = PAM_SYSTEM_ERR;
                        } else {
                                krc = PAM_SUCCESS;
                        }
                }
        }

                if (RC_OK && ((authtok == NULL) || (strlen(authtok) == 0))) {
                        prc = PAM_AUTHTOK_ERR;
                }

                if (RC_OK) {
                        memset(&creds, 0, sizeof(creds));
                        creds.client = principal;
                        creds.server = server;
                        krc = krb5_get_init_creds_password(context,
                                                           &creds,
                                                           principal,
                                                           (char*)old_authtok,
                                                           NULL,
                                                           NULL,
                                                           0,
                                                           PASSWORD_CHANGING_SERVICE,
                                                           &config->creds_opt);
                        if (krc == KRB5_SUCCESS) {
                                DEBUG("%s prepared for password change", user);
                        } else {
                                INFO("can't change password for %s: %d (%s)",
                                     user, krc, error_message(krc));
                        }
                }

                if (RC_OK) {
                        krc = krb5_change_password(context,
                                                   &creds,
                                                   authtok,
                                                   &result_code,
                                                   &result_code_string,
                                                   &result_string);
                        if ((krc == KRB5_SUCCESS) &&
                           (result_code == KRB5_KPASSWD_SUCCESS)) {
                                INFO("%s's %s password has been changed",
                                     user, config->banner);
                        } else {
                                INFO("changing %s's %s password failed: %*s "
                                     "(%d: %*s)",
                                     user, config->banner,
                                     result_string.length,
                                     result_string.data,
                                     result_code,
                                     result_code_string.length,
                                     result_code_string.data);
                        }
                }

        /* Clean up and return. */
        if (context != NULL) {
                krb5_free_context(context);
        }

}

